#include <stdio.h>

#include "allegro5/allegro.h"
#include "allegro5/allegro_image.h"
#include "allegro5/allegro_opengl.h"
#include "allegro5/allegro_primitives.h"

#include "GL/glu.h"
#include "math.h"
#include "signal.h"

#define MAXNUMBITMAPS 8

#ifdef IMX6
   #define al_create_display(x,y) al_setup_display_imx6(x, y)
   #define al_init()              al_init_imx6()
#endif

#ifndef WHERE
   #define WHERE() { printf("line %d\n", __LINE__); }
#endif

extern int              al_init_imx6();
extern ALLEGRO_DISPLAY *al_setup_display_imx6(int w, int h);

float ss=1.0;
float pos=0.0f;
bool  loop=true;

int SCREEN_W = 1024;
int SCREEN_H =  768;

void draw_flat_texture(GLuint id, float u , float v , float x, float y, float w, float h);

void sig_int(int s) { WHERE(); loop=false; }

// ** camera_3D_setup *****************************************************************
void camera_3D_setup()
{
   glMatrixMode(GL_PROJECTION);    // current matrix stack (for subsequent matrix operations) = GL_PROJECTION 
   glLoadIdentity();               // replace the current matrix with the identity matrix
   gluPerspective(35.0, (GLdouble)SCREEN_W / (GLdouble)SCREEN_H, 0.0, 2000.0);
}

// ** camera_2D_setup *****************************************************************
void camera_2D_setup()
{
   glMatrixMode(GL_PROJECTION);    // current matrix stack (for subsequent matrix operations) = GL_PROJECTION
   glLoadIdentity();               // replace the current matrix with the identity matrix
   glOrtho(0.0, (GLdouble)SCREEN_W, (GLdouble)SCREEN_H, 0.0, 0.0, 1.0);
}

ALLEGRO_BITMAP *bmp;
GLuint          oti;
GLuint          ogl_tex;


// ** main *****************************************************************
int main(int argc, char *argv[])
{
   ALLEGRO_DISPLAY     *display = NULL;
   ALLEGRO_EVENT_QUEUE *eventq  = NULL;
   ALLEGRO_TIMER       *timer   = NULL;

   float fps  = 60.0f;
   bool  draw = false;

   signal(SIGINT,  sig_int);
   signal(SIGTERM, sig_int);

   GLUquadricObj *quad_1 = gluNewQuadric();

   // ** initialise graphics, extensions, etc
   al_init();
   al_init_image_addon();
   al_init_primitives_addon();
   al_install_keyboard();

   // ** setup display
   al_set_new_display_flags   (ALLEGRO_OPENGL);
   al_set_new_display_option  (ALLEGRO_VSYNC, 1, ALLEGRO_SUGGEST);
   display = al_create_display(SCREEN_W, SCREEN_H);

   eventq = al_create_event_queue();
   timer  = al_create_timer(1.0f/fps);

   // load bitmaps
   al_set_new_bitmap_flags(ALLEGRO_MIPMAP | ALLEGRO_MIN_LINEAR);

   bmp = al_load_bitmap("img_2.jpg");

   ogl_tex = al_get_opengl_texture(bmp); // store OPENGL texture id

   // ** register event sources
   al_register_event_source(eventq, al_get_display_event_source (display));
   al_register_event_source(eventq, al_get_timer_event_source   (timer));
   al_register_event_source(eventq, al_get_keyboard_event_source());

   al_start_timer(timer);


   // ** camera setup

   // 2D camera
   camera_2D_setup();

   // default flat background
   al_draw_bitmap(bmp, 0.0f, 0.0f, 0);

   // 3D camera
   camera_3D_setup();


   int color = 0;
   // ** main loop
   while (loop)
   {
      ALLEGRO_EVENT event;
      al_wait_for_event(eventq, &event);

      switch(event.type)
      {
         case ALLEGRO_EVENT_KEY_DOWN:
         case ALLEGRO_EVENT_DISPLAY_CLOSE:
            loop = false; break;
         case ALLEGRO_EVENT_TIMER:
            color = (int)(fmod(al_get_timer_speed(timer)*al_get_timer_count(timer)*64.0 , 256.0));
            draw = true;
            break;
         default: break;
      }

      if ((draw == true) && (al_event_queue_is_empty(eventq)))
      {



         // draw something onto texture - *** this is very detrimental to everything(!) ***
         //if (0)
         // draw onto texture
         al_set_target_bitmap(bmp);
///         al_clear_to_color(al_map_rgb(0,255,0));
         al_clear_to_color(al_map_rgb(color , color , color));
         
         ogl_tex = al_get_opengl_texture(bmp);
         
         al_set_target_backbuffer(display);
         al_clear_to_color(al_map_rgb(0,0,0));
         
         al_draw_bitmap(bmp , 0 , 0 , 0);
         
         al_draw_filled_rectangle(SCREEN_W - 50 , SCREEN_H - 50 , SCREEN_W , SCREEN_H , al_map_rgb(255,255,255));
         
         //al_draw_rectangle(0, 0, pos*2, pos*2, al_map_rgb(255,255,255), 5);

         // try setting up cameras again
         //camera_2D_setup(); 
         //camera_3D_setup();

         // ** 
         glEnable(GL_DEPTH_TEST);
         glEnable(GL_TEXTURE_2D);

///         glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // clear the depth buffer

///         glViewport(0 , 0 , SCREEN_W , SCREEN_H);
         
         camera_3D_setup();
         
         glMatrixMode(GL_TEXTURE);
         glLoadIdentity();
         glMatrixMode(GL_COLOR);
         glLoadIdentity();
         
         glMatrixMode(GL_MODELVIEW);    // set current matrix stack = GL_MODELVIEW
         glLoadIdentity();              // replace current matrix with the identity matrix


         gluLookAt(0.0f, 0.0f, 1650.0f, 0.0f, 0.0f, 0.0f, 0.0, 1.0f, 0.0f); 

         // ** draw (hopefully modified) flat image
         int iu = 0 , iv = 0;
         al_get_opengl_texture_position(bmp , &iu , &iv);
         
         float u = (float)iu/al_get_bitmap_width(bmp);
         float v = (float)iv/al_get_bitmap_height(bmp);
         draw_flat_texture(ogl_tex, u , v , -400, -300, 256, 192);

         // ** rotate camera position around 0,0,0
         pos += 1.0; pos = fmod(pos, 360.0);
         glRotatef(pos/3.14,  2.0f, 0.0f, 0.0f);
         glRotatef(pos,       0.0f, 2.0f, 0.0f);


         // ** draw sphere
         glRotatef   (-80.0f, 1.0f, 0.0f, 0.0f);      // rotate X - fixed
         glRotatef   (-15.0f, 0.0f, 1.0f, 0.0f);      // rotate Y - fixed
         glRotatef   (-20.0f, 0.0f, 0.0f, 1.0f);      // rotate Z - fixed

         glBindTexture  (GL_TEXTURE_2D, ogl_tex);
         glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
         glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

         gluQuadricTexture(quad_1, GL_TRUE);          // start texture coordinate generation
         gluSphere        (quad_1, 70.0f, 28, 28);    // radius, slices, stacks
         gluQuadricTexture(quad_1, GL_FALSE);         // halt  texture coordinate generation

         glDisable(GL_TEXTURE_2D);
         glDisable(GL_DEPTH_TEST);
         glFlush();

         draw = false;
         al_flip_display();
      }
   }

   al_destroy_bitmap(bmp);

   glDeleteTextures (1, &oti);
   gluDeleteQuadric (quad_1);
   
   return 0;
}

// ** draw_flat_texture *****************************************************************
void draw_flat_texture(GLuint id, float u , float v , float x, float y, float w, float h)
{
glBindTexture  (GL_TEXTURE_2D, id);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

///glPushMatrix(); // push on stack

glBegin(GL_QUADS);
glTexCoord2f(u, v); glVertex3f(x,   y, 1.0);
glTexCoord2f(u, v+1); glVertex3f(x,   y + h, 1.0);
glTexCoord2f(u+1, v+1); glVertex3f(x + w,   y + h, 1.0);
glTexCoord2f(u+1, v); glVertex3f(x + w,   y, 1.0);
glEnd();

///glPopMatrix();  // pop off stack
}


